#include <DHT.h>
#include <LiquidCrystal.h>
#include <Keypad.h>

// === DHT11 Setup ===
#define DHTPIN 12
#define DHTTYPE DHT11
DHT dht(DHTPIN, DHTTYPE);

// === LCD ===
LiquidCrystal lcd(22, 23, 24, 25, 26, 27);

// === RGB LED 1 for DHT11 ===
const int redPin1 = 9;
const int greenPin1 = 10;
const int bluePin1 = 11;

// === RGB LED 2 for Soil Moisture ===
const int redPin2 = 5;
const int greenPin2 = 6;
const int bluePin2 = 7;

// === Soil Moisture ===
const int soilAnalogPin = A0;
const int soilDigitalPin = 8;

// === Sound Sensor ===
const int soundDigitalPin = 3;
const int redLED = 2;
const int led13 = 13;

// === Keypad ===
const byte ROWS = 4;
const byte COLS = 4;
char keys[ROWS][COLS] = {
  {'D','C','B','A'},
  {'#','9','6','3'},
  {'0','8','5','2'},
  {'*','7','4','1'}
};
byte rowPins[ROWS] = {31, 33, 35, 37};
byte colPins[COLS] = {39, 41, 43, 45};
Keypad keypad = Keypad(makeKeymap(keys), rowPins, colPins, ROWS, COLS);

// === Display Mode ===
int displayMode = 1;
unsigned long lastUpdate = 0;
const unsigned long updateInterval = 2000;

void setup() {
  Serial.begin(9600);
  dht.begin();
  lcd.begin(16, 2);

  // Set RGB pins
  pinMode(redPin1, OUTPUT);
  pinMode(greenPin1, OUTPUT);
  pinMode(bluePin1, OUTPUT);

  pinMode(redPin2, OUTPUT);
  pinMode(greenPin2, OUTPUT);
  pinMode(bluePin2, OUTPUT);

  pinMode(soilDigitalPin, INPUT);
  pinMode(soundDigitalPin, INPUT);
  pinMode(redLED, OUTPUT);
  pinMode(led13, OUTPUT);

  lcd.print("System Ready");
  delay(1500);
  lcd.clear();
}

void loop() {
  // === Real-Time Sound Sensor Blinking ===
  int soundValue = digitalRead(soundDigitalPin);
  digitalWrite(redLED, soundValue);
  digitalWrite(led13, soundValue);

  // === Keypad Handling ===
  char key = keypad.getKey();
  if (key != NO_KEY) {
    if (key == '1') {
      displayMode = 1;
      lcd.clear();
    } else if (key == '2') {
      displayMode = 2;
      lcd.clear();
    }
  }

  // === Sensor Update Every 2 Seconds ===
  unsigned long currentMillis = millis();
  if (currentMillis - lastUpdate >= updateInterval) {
    lastUpdate = currentMillis;

    float temp = dht.readTemperature(true); // Fahrenheit
    float humid = dht.readHumidity();
    int soilRaw = analogRead(soilAnalogPin);
    int soilPercent = map(soilRaw, 1023, 300, 0, 100);
    soilPercent = constrain(soilPercent, 0, 100);
    int soilState = digitalRead(soilDigitalPin);

    if (displayMode == 1) {
      lcd.setCursor(0, 0);
      lcd.print("Soil Moisture");
      lcd.setCursor(0, 1);
      lcd.print("Level: ");
      lcd.print(soilPercent);
      lcd.print("%");

      // RGB LED for Soil Moisture
      if (soilPercent < 50) setColor2(255, 0, 0);       // Red = dry
      else if (soilPercent <= 75) setColor2(0, 255, 0); // Green = moist
      else setColor2(0, 0, 255);                        // Blue = wet
    }

    else if (displayMode == 2) {
      lcd.setCursor(0, 0);
      lcd.print("Temp: ");
      lcd.print(temp);
      lcd.print((char)223); // degree symbol
      lcd.print("F");

      lcd.setCursor(0, 1);
      lcd.print("Humidity: ");
      lcd.print(humid);
      lcd.print("%");

      // RGB LED for Temperature
      if (temp < 70) setColor1(0, 0, 255);         // Blue = cold
      else if (temp > 75) setColor1(255, 0, 0);    // Red = hot
      else setColor1(0, 255, 0);                   // Green = normal
    }

    // === Serial Output ===
    Serial.print("Temp: "); Serial.print(temp);
    Serial.print(" | Humid: "); Serial.print(humid);
    Serial.print(" | Soil Raw: "); Serial.print(soilRaw);
    Serial.print(" | Soil %: "); Serial.print(soilPercent);
    Serial.print(" | Soil State: "); Serial.print(soilState == HIGH ? "Dry" : "Wet");
    Serial.print(" | Sound D0: "); Serial.println(soundValue);
  }
}

void setColor1(int r, int g, int b) {
  analogWrite(redPin1, r);
  analogWrite(greenPin1, g);
  analogWrite(bluePin1, b);
}

void setColor2(int r, int g, int b) {
  analogWrite(redPin2, r);
  analogWrite(greenPin2, g);
  analogWrite(bluePin2, b);
}
